// Display Mars central meridian and list of features,
// as html/cgi or plaintext.
// Copyright 2001 by Akkana Peck.

#include "time.h"
#include "string.h"
#include "ctype.h"

#include "features.h"

#ifdef HTML
#define PRINTHEAD printf("Content-type: text/html\n\n<html>\n<head>\n<title>Mars Ephemeris</title>\n</head>\n");
#define PRINTBR   printf("<br>\n")
#define PRINTFOOT printf("\n</body>\n</html>\n");
#else
#define PRINTHEAD
#define PRINTBR
#define PRINTFOOT
#endif

int parseMonth(char* mon);
void PrintFeaturesVisible(int meridian);

int main(int argc, char **argv)
{
    time_t sec = 0;
    long afterOpposition = 992476800;
    int meridian;

    PRINTHEAD;

    printf("Mars 2001\n");
    PRINTBR;

    if (argc <= 1)
    {
        time(&sec);
#ifdef DEBUG
        printf("Time sez it's %ld, or '%s'\n", sec, ctime(&sec));
#endif
        sec -= afterOpposition;
    }
    else if (argc == 2 && isdigit(argv[1][0]))
    {
        sec = atoi(argv[1]);
        // Don't convert to epoch, assume user knows what they're doing.
    }
    else if (argc == 4 || argc == 5)
    {
        struct tm then;
        char* min;
        char* hour;
        if (isdigit(argv[1][0]))
            then.tm_mon = atoi(argv[1]) - 1;
        else
            then.tm_mon = parseMonth(argv[1]);

        then.tm_mday = atoi(argv[2]);

        if (argc == 4) {
            then.tm_year = 101;
            hour = argv[3];
        }
        else {
            then.tm_year = atoi(argv[3]);
            // A struct tm year is actually year - 1900, hah.
            if (then.tm_year > 200)
                then.tm_year -= 1900;
            hour = argv[4];
        }

        min = strchr(hour, ':');
        if (min) {
            *(min++) = '\0';
            then.tm_min = atoi(min);
        }
        else then.tm_min = 0;
        then.tm_hour = atoi(hour);

        then.tm_sec = 0;
        then.tm_isdst = -1;

        sec = mktime(&then);
#ifdef DEBUG
        printf("%d / %d / %d %d:%02d:%02d %s.\n",
               then.tm_mon+1, then.tm_mday, then.tm_year,
               then.tm_hour, then.tm_min, then.tm_sec, tzname);
        printf("asctime says '%s'\n", asctime(&then));

        printf("mktime said: %ld sec\n", sec);
#endif /* DEBUG */
        sec -= afterOpposition;
    }
    else {
        printf("Usage: %s [seconds]|[month day [year] hh:mm]\n", argv[0]);
        return 1;
    }
    printf("Time to oposition: %ld sec\n", sec);
    meridian = GetCentralMeridian((long)sec);
    PRINTBR;
    printf("=======================================\n");
    PRINTBR;
    printf("======= Central Meridian: %3d =========\n", meridian);
    PRINTBR;
    printf("=======================================\n");
    PRINTBR;
    PrintFeaturesVisible(meridian);
    PRINTBR;

    PRINTFOOT;
}

int parseMonth(char* mon)
{
    if (!strncmp(mon, "Jan", 3))
        return 0;
    else if (!strncmp(mon, "Feb", 3))
        return 1;
    else if (!strncmp(mon, "Mar", 3))
        return 2;
    else if (!strncmp(mon, "Apr", 3))
        return 3;
    else if (!strncmp(mon, "May", 3))
        return 4;
    else if (!strncmp(mon, "Jun", 3))
        return 5;
    else if (!strncmp(mon, "Jul", 3))
        return 6;
    else if (!strncmp(mon, "Aug", 3))
        return 7;
    else if (!strncmp(mon, "Sep", 3))
        return 8;
    else if (!strncmp(mon, "Oct", 3))
        return 9;
    else if (!strncmp(mon, "Nov", 3))
        return 10;
    else if (!strncmp(mon, "Dec", 3))
        return 11;
}

// Print features currently visible, in about 15 lines x 30 cols.
void
PrintFeaturesVisible(int meridian)
{
    int NumFeatures = sizeof (Features) / sizeof (*Features);
    int startLong = (meridian - 90) % 360;
    int endLong = (meridian + 90) % 360;
    int i;
    int isVisible = 0;

    /*
    printf("Looking for features around %d, between %d and %d\n",
           meridian, startLong, endLong);
    */
    for (i=0; i < NumFeatures; ++i)
    {
        // See if it's time yet to toggle visibility on
        if (Features[i].longitude >= startLong
            && (i == 0 || Features[i-1].longitude < startLong))
        {
            //printf("Turning visibility on at %s\n", Features[i].name);
            isVisible = 1;
        }
        // Or off (in which case we're done)
        else if (isVisible && Features[i].longitude > startLong+180)
        {
            //printf("Returning from first loop in %s\n", Features[i].name);
            return;
        }
        if (isVisible)
            printf("%s (%d, %d) %s\n", Features[i].name,
                   Features[i].latitude, Features[i].longitude,
                   (Features[i].lightP ? "(light)" : "(dark)"));
    }
    // Wrap around: if we didn't return from the last loop,
    // then we have some wraparound to do and isVisible should be on.
    //printf("Wrapping around\n");
    if (!isVisible)
        printf("Uh-oh, wrapped around but !isVisible\n");
    for (i=0; i < NumFeatures; ++i)
    {
        if (Features[i].longitude > endLong)
        {
            //printf("Returning from second loop at %s\n", Features[i].name);
            return;
        }
        printf("%s (%d, %d) %s\n", Features[i].name,
               Features[i].latitude, Features[i].longitude,
               (Features[i].lightP ? "(light)" : "(dark)"));
    }
}


